/*
 * Copyright 2020 Rockchip Electronics S.LSI Co. LTD
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
*/

#ifndef __TOYBRICK_LOG_H__
#define __TOYBRICK_LOG_H__

#if __ANDROID__
#include <android/log.h>

#ifndef LOG_TAG
#define LOG_TAG "toybrick"
#endif
#define toybrick_debug(tag, fmt, ...) __android_log_print(ANDROID_LOG_DEBUG, tag, fmt, ## __VA_ARGS__)
#define toybrick_info(tag, fmt, ...) __android_log_print(ANDROID_LOG_INFO, tag, fmt, ## __VA_ARGS__)
#define toybrick_warn(tag, fmt, ...) __android_log_print(ANDROID_LOG_WARNING, tag, fmt, ## __VA_ARGS__)
#define toybrick_err(tag, fmt, ...) __android_log_print(ANDROID_LOG_ERROR, tag, fmt, ## __VA_ARGS__)

#define ALOGD(...) __android_log_print(ANDROID_LOG_DEBUG, LOG_TAG, __VA_ARGS__)
#define ALOGI(...) __android_log_print(ANDROID_LOG_INFO, LOG_TAG, __VA_ARGS__)
#define ALOGW(...) __android_log_print(ANDROID_LOG_WARN, LOG_TAG, __VA_ARGS__)
#define ALOGE(...) __android_log_print(ANDROID_LOG_ERROR, LOG_TAG, __VA_ARGS__)

#else

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#define TOYBRICK_DEFAULT_MASK        0x0c

enum {
	TOYBRICK_LOG_DEBUG = (1 << 0),
	TOYBRICK_LOG_INFO = (1 << 1),
	TOYBRICK_LOG_WARNING = (1 << 2),
	TOYBRICK_LOG_ERROR = (1 << 3),
	TOYBRICK_LOG_PREFIX = (1 << 7),
};

#define toybrick_log(level, tag, fmt, ...)			\
	do {							\
		time_t timep;					\
		struct tm *p;					\
		char flag;					\
		int mask = TOYBRICK_DEFAULT_MASK;		\
		char *env = getenv("toybrick_log_mask");	\
		switch (level) {				\
		case TOYBRICK_LOG_DEBUG: flag = 'D'; break;	\
		case TOYBRICK_LOG_INFO: flag = 'I'; break;	\
		case TOYBRICK_LOG_WARNING: flag = 'W'; break;	\
		case TOYBRICK_LOG_ERROR: flag = 'E'; break;	\
		default: flag = 'U'; break;			\
		}						\
		if (env)					\
			sscanf(env, "%x", &mask);		\
		if (mask & level) {				\
			time(&timep);				\
			p = gmtime(&timep);			\
			fprintf(stderr, "[%02d-%02d %02d:%02d:%02d %c %s] " fmt, \
				p->tm_mon + 1, p->tm_mday, p->tm_hour, p->tm_min, p->tm_sec, \
				flag, tag, ## __VA_ARGS__); \
		}						\
	} while (0)


#define toybrick_dbg(tag, fmt, ...) toybrick_log(TOYBRICK_LOG_DEBUG, tag, fmt, ## __VA_ARGS__)
#define toybrick_info(tag, fmt, ...) toybrick_log(TOYBRICK_LOG_INFO, tag, fmt, ## __VA_ARGS__)
#define toybrick_warn(tag, fmt, ...) toybrick_log(TOYBRICK_LOG_WARNING, tag, fmt, ## __VA_ARGS__)
#define toybrick_err(tag, fmt, ...) toybrick_log(TOYBRICK_LOG_ERROR, tag, fmt, ## __VA_ARGS__)
#endif

#endif
